@@ -1,8 +1,13 @@
-{-# LANGUAGE PatternGuards, RecordWildCards, DeriveDataTypeable #-}
+{-# LANGUAGE PatternGuards, DeriveDataTypeable #-}
 {-# OPTIONS_GHC -fno-warn-missing-fields -fno-cse -O0 #-}
 
-module CmdLine(Cmd(..), cmdCpp, CppFlags(..), getCmd, cmdExtensions, cmdHintFiles, cmdUseColour, exitWithHelp, resolveFile) where
+module CmdLine(
+    Cmd(..), getCmd,
+    CppFlags(..), cmdCpp, cmdExtensions, cmdHintFiles, cmdUseColour,
+    exitWithHelp, resolveFile
+    ) where
 
+import Control.Monad.Extra
 import Data.Char
 import Data.List
 import System.Console.ANSI(hSupportsANSI)
@@ -16,32 +21,36 @@ import Language.Preprocessor.Cpphs
 import Language.Haskell.Exts(defaultParseMode, baseLanguage)
 import Language.Haskell.Exts.Extension
 import Data.Maybe
-import System.Environment
-import qualified System.Info.Extra as Info
+import System.Environment.Extra
+import System.Info.Extra
 
 import Util
 import Paths_hlint
 import Data.Version
 
+
 getCmd :: [String] -> IO Cmd
 getCmd args = withArgs (map f args) $ automatic =<< cmdArgsRun mode
     where f x = if x == "-?" || x == "--help" then "--help=all" else x
 
 
 automatic :: Cmd -> IO Cmd
-automatic CmdMain{..} = do
-    cmdDataDir <- if cmdDataDir == "" then getDataDir else return cmdDataDir
-    cmdPath <- return $ if null cmdPath then ["."] else cmdPath
-    cmdExtension <- return $ if null cmdExtension then ["hs", "lhs"] else cmdExtension
-    return CmdMain{..}
-automatic CmdGrep{..} = do
-    cmdPath <- return $ if null cmdPath then ["."] else cmdPath
-    cmdExtension <- return $ if null cmdExtension then ["hs", "lhs"] else cmdExtension
-    return CmdGrep{..}
-automatic CmdTest{..} = do
-    cmdDataDir <- if cmdDataDir == "" then getDataDir else return cmdDataDir
-    return CmdTest{..}
-automatic x = return x
+automatic cmd = case cmd of
+    CmdMain{} -> dataDir $ path $ extension cmd
+    CmdGrep{} -> return $ path $ extension cmd
+    CmdTest{} -> dataDir cmd
+    _ -> return cmd
+    where
+        path cmd = if null $ cmdPath cmd then cmd{cmdPath=["."]} else cmd
+        extension cmd = if null $ cmdExtension cmd then cmd{cmdExtension=["hs","lhs"]} else cmd
+        dataDir cmd
+            | cmdDataDir cmd  /= "" = return cmd
+            | otherwise = do
+                x <- getDataDir
+                b <- doesDirectoryExist x
+                if b then return cmd{cmdDataDir=x} else do
+                    exe <- getExecutablePath
+                    return cmd{cmdDataDir = takeDirectory exe </> "data"}
 
 
 exitWithHelp :: IO a
@@ -66,7 +75,7 @@ data ColorMode
 
 
 instance Default ColorMode where
-  def = if Info.isWindows then Never else Auto
+  def = if isWindows then Never else Auto
 
 
 data Cmd
@@ -81,11 +90,10 @@ data Cmd
         ,cmdShowAll :: Bool              -- ^ display all skipped items
         ,cmdExtension :: [String]        -- ^ extensions
         ,cmdLanguage :: [String]      -- ^ the extensions (may be prefixed by "No")
-        ,cmdUtf8 :: Bool
-        ,cmdEncoding :: String         -- ^ the text encoding
         ,cmdCross :: Bool                -- ^ work between source files, applies to hints such as duplicate code between modules
         ,cmdFindHints :: [FilePath]      -- ^ source files to look for hints in
         ,cmdDataDir :: FilePath          -- ^ the data directory
+        ,cmdDefault :: Bool              -- ^ Print a default file to stdout
         ,cmdPath :: [String]
         ,cmdCppDefine :: [String]
         ,cmdCppInclude :: [FilePath]
@@ -106,8 +114,6 @@ data Cmd
         ,cmdPattern :: String
         ,cmdExtension :: [String]        -- ^ extensions
         ,cmdLanguage :: [String]      -- ^ the extensions (may be prefixed by "No")
-        ,cmdUtf8 :: Bool
-        ,cmdEncoding :: String         -- ^ the text encoding
         ,cmdPath :: [String]
         ,cmdCppDefine :: [String]
         ,cmdCppInclude :: [FilePath]
@@ -143,11 +149,10 @@ mode = cmdArgsMode $ modes
         ,cmdShowAll = nam "show" &= help "Show all ignored ideas"
         ,cmdExtension = nam "extension" &= typ "EXT" &= help "File extensions to search (default hs/lhs)"
         ,cmdLanguage = nam_ "language" &= name "X" &= typ "EXTENSION" &= help "Language extensions (Arrows, NoCPP)"
-        ,cmdUtf8 = nam "utf8" &= help "Use UTF-8 text encoding"
-        ,cmdEncoding = nam_ "encoding" &= typ "ENCODING" &= help "Choose the text encoding"
         ,cmdCross = nam_ "cross" &= help "Work between modules"
         ,cmdFindHints = nam "find" &= typFile &= help "Find hints in a Haskell file"
-        ,cmdDataDir = nam "datadir" &= typDir &= help "Override the data directory"
+        ,cmdDataDir = nam_ "datadir" &= typDir &= help "Override the data directory"
+        ,cmdDefault = nam "default" &= help "Print a default file to stdout"
         ,cmdPath = nam "path" &= help "Directory in which to search for files"
         ,cmdCppDefine = nam_ "cpp-define" &= typ "NAME[=VALUE]" &= help "CPP #define"
         ,cmdCppInclude = nam_ "cpp-include" &= typDir &= help "CPP include path"
@@ -174,19 +179,40 @@ mode = cmdArgsMode $ modes
         ,cmdTempDir = nam_ "tempdir" &= help "Where to put temporary files (not cleaned up)"
         } &= explicit &= name "test"
         &= details ["HLint gives hints on how to improve Haskell code."
-                 ,""
-                 ,"To check all Haskell files in 'src' and generate a report type:"
-                 ,"  hlint src --report"]
+                   ,""
+                   ,"To check all Haskell files in 'src' and generate a report type:"
+                   ,"  hlint src --report"]
     ,CmdHSE
         {} &= explicit &= name "hse"
     ] &= program "hlint" &= verbosity
-    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2016")
+    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2017")
     where
         nam xs = nam_ xs &= name [head xs]
         nam_ xs = def &= explicit &= name xs
 
+-- | Where should we find the configuration files?
+--   * If someone passes cmdWithHints, only look at files they explicitly request
+--   * If someone passes an explicit hint name, automatically merge in data/hlint.yaml
+--   We want more important hints to go last, since they override
 cmdHintFiles :: Cmd -> IO [FilePath]
-cmdHintFiles cmd = mapM (getHintFile $ cmdDataDir cmd) $ cmdGivenHints cmd ++ ["HLint" | null (cmdGivenHints cmd) && null (cmdWithHints cmd)]
+cmdHintFiles cmd = do
+    let explicit1 = [cmdDataDir cmd </> "hlint.yaml" | null $ cmdWithHints cmd]
+    let explicit2 = cmdGivenHints cmd
+    bad <- filterM (notM . doesFileExist) $ explicit1 ++ explicit2
+    when (bad /= []) $
+        fail $ unlines $ "Failed to find requested hint files:" : map ("  "++) bad
+    if cmdWithHints cmd /= [] then return $ explicit1 ++ explicit2 else do
+        -- we follow the stylish-haskell config file search policy
+        -- 1) current directory or its ancestors; 2) home directory
+        curdir <- getCurrentDirectory
+        home <- getHomeDirectory
+        b <- doesFileExist ".hlint.yaml"
+        implicit <- findM doesFileExist $
+            "HLint.hs" : -- the default in HLint 1.*
+            map (</> ".hlint.yaml") (ancestors curdir ++ [home]) -- to match Stylish Haskell
+        return $ explicit1 ++ maybeToList implicit ++ explicit2
+    where
+        ancestors = init . map joinPath . reverse . inits . splitPath
 
 cmdExtensions :: Cmd -> (Language, [Extension])
 cmdExtensions = getExtensions . cmdLanguage
@@ -216,7 +242,11 @@ cmdUseColour cmd = case cmdColor cmd of
 x <\> y = x </> y
 
 
-resolveFile :: Cmd -> Maybe FilePath -> FilePath -> IO [FilePath]
+resolveFile
+    :: Cmd
+    -> Maybe FilePath -- ^ Temporary file
+    -> FilePath       -- ^ File to resolve, may be "-" for stdin
+    -> IO [FilePath]
 resolveFile cmd = getFile (cmdPath cmd) (cmdExtension cmd)
 
 
@@ -224,7 +254,7 @@ getFile :: [FilePath] -> [String] -> Maybe FilePath -> FilePath -> IO [FilePath]
 getFile path _ (Just tmpfile) "-" =
   getContents >>= writeFile tmpfile >> return [tmpfile]
 getFile path _ Nothing "-" = return ["-"]
-getFile [] exts _ file = error $ "Couldn't find file: " ++ file
+getFile [] exts _ file = exitMessage $ "Couldn't find file: " ++ file
 getFile (p:ath) exts t file = do
     isDir <- doesDirectoryExist $ p <\> file
     if isDir then do
@@ -258,20 +288,6 @@ getModule path exts x | not (any isSpace x) && all isMod xs = f exts
 getModule _ _ _ = return Nothing
 
 
-getHintFile :: FilePath -> FilePath -> IO FilePath
-getHintFile _ "-" = return "-"
-getHintFile dataDir x = do
-        let poss = nub $ concat [x : [x <.> "hs" | takeExtension x /= ".hs"] | x <- [x,dataDir </> x]]
-        f poss poss
-    where
-        f o [] = error $ unlines $ [
-            "Couldn't find file: " ++ x,
-            "Tried with:"] ++ map ("  "++) o
-        f o (x:xs) = do
-            b <- doesFileExist x
-            if b then return x else f o xs
-
-
 getExtensions :: [String] -> (Language, [Extension])
 getExtensions args = (lang, foldl f (if null langs then defaultExtensions else []) exts)
     where
@@ -282,7 +298,7 @@ getExtensions args = (lang, foldl f (if null langs then defaultExtensions else [
         f a "Haskell98" = []
         f a ('N':'o':x) | Just x <- readExtension x = delete x a
         f a x | Just x <- readExtension x = x : delete x a
-        f a x = error $ "Unknown extension: " ++ x
+        f a x = exitMessageImpure $ "Unknown extension: " ++ x
 
 
 readExtension :: String -> Maybe Extension
