@@ -1,7 +1,8 @@
 
 module Config.Type(
     Severity(..), Classify(..), HintRule(..), Note(..), Setting(..),
-    defaultHintName, isUnifyVar, showNotes,
+    Restrict(..), RestrictType(..),
+    defaultHintName, isUnifyVar, showNotes, getSeverity, getRestrictType
     ) where
 
 import HSE.All
@@ -10,6 +11,23 @@ import Data.List.Extra
 import Prelude
 
 
+getSeverity :: String -> Maybe Severity
+getSeverity "ignore" = Just Ignore
+getSeverity "warn" = Just Warning
+getSeverity "warning" = Just Warning
+getSeverity "suggest" = Just Suggestion
+getSeverity "suggestion" = Just Suggestion
+getSeverity "error" = Just Error
+getSeverity "hint" = Just Suggestion
+getSeverity _ = Nothing
+
+getRestrictType :: String -> Maybe RestrictType
+getRestrictType "modules" = Just RestrictModule
+getRestrictType "extensions" = Just RestrictExtension
+getRestrictType "flags" = Just RestrictFlag
+getRestrictType "functions" = Just RestrictFunction
+getRestrictType _ = Nothing
+
 defaultHintName :: String
 defaultHintName = "Use alternative"
 
@@ -46,7 +64,7 @@ instance Show Note where
     show IncreasesLaziness = "increases laziness"
     show DecreasesLaziness = "decreases laziness"
     show (RemovesError x) = "removes error " ++ x
-    show (ValidInstance x y) = "requires a valid " ++ x ++ " instance for " ++ y
+    show (ValidInstance x y) = "requires a valid `" ++ x ++ "` instance for `" ++ y ++ "`"
     show (Note x) = x
 
 
@@ -76,10 +94,21 @@ data HintRule = HintRule
     }
     deriving Show
 
+data RestrictType = RestrictModule | RestrictExtension | RestrictFlag | RestrictFunction deriving (Show,Eq,Ord)
+
+data Restrict = Restrict
+    {restrictType :: RestrictType
+    ,restrictDefault :: Bool
+    ,restrictName :: [String]
+    ,restrictAs :: [String] -- for RestrictModule only, what you can import it as
+    ,restrictWithin :: [(String, String)]
+    } deriving Show
+
 data Setting
     = SettingClassify Classify
     | SettingMatchExp HintRule
+    | SettingRestrict Restrict
+    | SettingArgument String -- ^ Extra command-line argument
     | Builtin String -- use a builtin hint set
     | Infix Fixity
       deriving Show
-
