@@ -1,18 +1,23 @@
-{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE FlexibleContexts, ViewPatterns #-}
 
 module HSE.Util(module HSE.Util) where
 
 import Control.Monad
 import Data.List
 import Data.Maybe
+import Data.Data hiding (Fixity)
 import System.FilePath
 import HSE.Type
+import Data.Functor
 import Prelude
 
 
 ---------------------------------------------------------------------
 -- ACCESSOR/TESTER
 
+ellipses :: QName S
+ellipses = UnQual an $ Ident an "..." -- Must be an Ident, not a Symbol
+
 opExp :: QOp S -> Exp_
 opExp (QVarOp s op) = Var s op
 opExp (QConOp s op) = Con s op
@@ -83,7 +88,6 @@ isVar Var{} = True; isVar _ = False
 isCon Con{} = True; isCon _ = False
 isApp App{} = True; isApp _ = False
 isInfixApp InfixApp{} = True; isInfixApp _ = False
-isList List{} = True; isList _ = False
 isAnyApp x = isApp x || isInfixApp x
 isParen Paren{} = True; isParen _ = False
 isIf If{} = True; isIf _ = False
@@ -109,6 +113,8 @@ isRecStmt RecStmt{} = True; isRecStmt _ = False
 isClsDefSig ClsDefSig{} = True; isClsDefSig _ = False
 isTyBang TyBang{} = True; isTyBang _ = False
 isLCase LCase{} = True; isLCase _ = False
+isTupleSection TupleSection{} = True; isTupleSection _ = False
+isString String{} = True; isString _ = False
 
 isSection LeftSection{} = True
 isSection RightSection{} = True
@@ -123,10 +129,10 @@ unqual :: QName S -> QName S
 unqual (Qual an _ x) = UnQual an x
 unqual x = x
 
-fromQual :: QName S -> Name S
-fromQual (Qual _ _ x) = x
-fromQual (UnQual _ x) = x
-fromQual x = error $ "HSE.Util.fromQual, not a name: " ++ prettyPrint x
+fromQual :: QName a -> Maybe (Name a)
+fromQual (Qual _ _ x) = Just x
+fromQual (UnQual _ x) = Just x
+fromQual _ = Nothing
 
 isSpecial :: QName S -> Bool
 isSpecial Special{} = True; isSpecial _ = False
@@ -176,6 +182,12 @@ isWHNF _ = False
 ---------------------------------------------------------------------
 -- HSE FUNCTIONS
 
+isKindHash :: Type_ -> Bool
+isKindHash (TyParen _ x) = isKindHash x
+isKindHash (TyApp _ x _) = isKindHash x
+isKindHash (TyCon _ (fromQual -> Just (Ident _ s))) = "#" `isSuffixOf`  s
+isKindHash _ = False
+
 
 getEquations :: Decl s -> [Decl s]
 getEquations (FunBind s xs) = map (FunBind s . (:[])) xs
@@ -250,15 +262,15 @@ transformAppsM f x = f =<< descendAppsM (transformAppsM f) x
 ---------------------------------------------------------------------
 -- UNIPLATE FUNCTIONS
 
-universeS :: Biplate x (f S) => x -> [f S]
+universeS :: (Data x, Data (f S)) => x -> [f S]
 universeS = universeBi
 
-childrenS :: Biplate x (f S) => x -> [f S]
+childrenS :: (Data x, Data (f S)) => x -> [f S]
 childrenS = childrenBi
 
 
 -- return the parent along with the child
-universeParentExp :: Biplate a Exp_ => a -> [(Maybe (Int, Exp_), Exp_)]
+universeParentExp :: Data a => a -> [(Maybe (Int, Exp_), Exp_)]
 universeParentExp xs = concat [(Nothing, x) : f x | x <- childrenBi xs]
     where f p = concat [(Just (i,p), c) : f c | (i,c) <- zip [0..] $ children p]
 
@@ -272,9 +284,6 @@ showSrcLoc (SrcLoc file line col) = take 1 file ++ f (drop 1 file) ++ ":" ++ sho
           f (x:xs) = x : f xs
           f [] = []
 
-toSrcSpan :: SrcSpanInfo -> SrcSpan
-toSrcSpan (SrcSpanInfo x _) = x
-
 nullSrcLoc :: SrcLoc
 nullSrcLoc = SrcLoc "" 0 0
 
@@ -302,6 +311,9 @@ notElem_ x = not . elem_ x
 nub_ :: (Annotated f, Eq (f ())) => [f S] -> [f S]
 nub_ = nubBy (=~=)
 
+delete_ :: (Annotated f, Eq (f ())) => f S -> [f S] -> [f S]
+delete_ = deleteBy (=~=)
+
 intersect_ :: (Annotated f, Eq (f ())) => [f S] -> [f S] -> [f S]
 intersect_ = intersectBy (=~=)
 
@@ -325,3 +337,6 @@ getFixity (InfixDecl sl a mp ops) = [Fixity (void a) (fromMaybe 9 mp) (UnQual ()
     where f (VarOp _ x) = x
           f (ConOp _ x) = x
 getFixity _ = []
+
+toInfixDecl :: Fixity -> Decl ()
+toInfixDecl (Fixity a b c) = InfixDecl () a (Just b) $ maybeToList $ VarOp () <$> fromQual c
