@@ -1,3 +1,4 @@
+{-# LANGUAGE RecordWildCards #-}
 {-
     Suggest removal of unnecessary extensions
     i.e. They have {-# LANGUAGE RecursiveDo #-} but no mdo keywords
@@ -83,6 +84,20 @@ foo = 12.3e2
 foo = id --
 {-# LANGUAGE NumDecimals #-} \
 foo = 12.345e2 --
+{-# LANGUAGE TupleSections #-} \
+main = map (,1,2) xs
+{-# LANGUAGE TupleSections #-} \
+main = id --
+{-# LANGUAGE OverloadedStrings #-} \
+main = "test"
+{-# LANGUAGE OverloadedStrings #-} \
+main = id --
+{-# LANGUAGE DeriveAnyClass #-} \
+main = id --
+{-# LANGUAGE DeriveAnyClass #-} \
+data Foo = Foo deriving Bob
+{-# LANGUAGE DeriveAnyClass #-} \
+data Foo a = Foo a deriving (Eq,Data,Functor) --
 </TEST>
 -}
 
@@ -93,11 +108,14 @@ import Hint.Type
 import Data.Maybe
 import Data.List.Extra
 import Data.Ratio
+import Data.Data
 import Refact.Types
+import Data.Monoid
+import Prelude
 
 
 extensionsHint :: ModuHint
-extensionsHint _ x = [rawIdea Warning "Unused LANGUAGE pragma" (toSrcSpan sl)
+extensionsHint _ x = [rawIdea Warning "Unused LANGUAGE pragma" (srcInfoSpan sl)
           (prettyPrint o) (Just newPragma)
           (warnings old new) [refact]
     | not $ used TemplateHaskell x -- if TH is on, can use all other extensions programmatically
@@ -120,14 +138,26 @@ warnings old new | wildcards `elem` old && wildcards `notElem` new = [Note "you
 warnings _ _ = []
 
 
--- | Classes that don't work with newtype deriving
-noNewtypeDeriving :: [String]
-noNewtypeDeriving = ["Read","Show","Data","Typeable","Generic","Generic1"]
+deriveHaskell = ["Eq","Ord","Enum","Ix","Bounded","Read","Show"]
+deriveGenerics = ["Data","Typeable","Generic","Generic1","Lift"]
+deriveCategory = ["Functor","Foldable","Traversable"]
+
+-- | Classes that can't require newtype deriving
+noGeneralizedNewtypeDeriving =
+    delete "Enum" deriveHaskell ++ -- Enum can't always be derived on a newtype
+    deriveGenerics -- Generics stuff can't newtype derive since it has the ctor in it
+
+-- | Classes that can't require DeriveAnyClass
+noDeriveAnyClass = deriveHaskell ++ deriveGenerics ++ deriveCategory
 
 
 usedExt :: Extension -> Module_ -> Bool
 usedExt (EnableExtension x) = used x
 usedExt (UnknownExtension "NumDecimals") = hasS isWholeFrac
+usedExt (UnknownExtension "DeriveLift") = hasDerive ["Lift"]
+usedExt (UnknownExtension "DeriveAnyClass") =
+    any (`notElem` noDeriveAnyClass) .
+    (\Derives{..} -> derivesNewType ++ derivesData) . derives
 usedExt _ = const True
 
 
@@ -167,8 +197,12 @@ used DeriveFunctor = hasDerive ["Functor"]
 used DeriveFoldable = hasDerive ["Foldable"]
 used DeriveTraversable = hasDerive ["Traversable"]
 used DeriveGeneric = hasDerive ["Generic","Generic1"]
-used GeneralizedNewtypeDeriving = any (`notElem` noNewtypeDeriving) . fst . derives
+used GeneralizedNewtypeDeriving =
+    any (`notElem` noGeneralizedNewtypeDeriving) .
+    (\Derives{..} -> derivesNewType ++ derivesStandalone) . derives
 used LambdaCase = hasS isLCase
+used TupleSections = hasS isTupleSection
+used OverloadedStrings = hasS isString
 used Arrows = hasS f
     where f Proc{} = True
           f LeftArrApp{} = True
@@ -185,30 +219,39 @@ used x = usedExt $ UnknownExtension $ show x
 
 
 hasDerive :: [String] -> Module_ -> Bool
-hasDerive want m = any (`elem` want) $ new ++ dat
-    where (new,dat) = derives m
+hasDerive want m = any (`elem` want) $ derivesNewType ++ derivesData ++ derivesStandalone
+    where Derives{..} = derives m
 
 
+data Derives = Derives
+    {derivesNewType :: [String]
+    ,derivesData :: [String]
+    ,derivesStandalone :: [String]
+    }
+instance Monoid Derives where
+    mempty = Derives [] [] []
+    mappend (Derives x1 x2 x3) (Derives y1 y2 y3) =
+        Derives (x1++y1) (x2++y2) (x3++y3)
+
 -- | What is derived on newtype, and on data type
 --   'deriving' declarations may be on either, so we approximate as both newtype and data
-derives :: Module_ -> ([String],[String])
-derives m = concatUnzip $ map decl (childrenBi m) ++ map idecl (childrenBi m)
+derives :: Module_ -> Derives
+derives m = mconcat $ map decl (childrenBi m) ++ map idecl (childrenBi m)
     where
-        idecl :: InstDecl S -> ([String], [String])
+        idecl :: InstDecl S -> Derives
         idecl (InsData _ dn _ _ ds) = g dn ds
         idecl (InsGData _ dn _ _ _ ds) = g dn ds
-        idecl _ = ([], [])
+        idecl _ = mempty
 
-        decl :: Decl_ -> ([String], [String])
+        decl :: Decl_ -> Derives
         decl (DataDecl _ dn _ _ _ ds) = g dn ds
         decl (GDataDecl _ dn _ _ _ _ ds) = g dn ds
         decl (DataInsDecl _ dn _ _ ds) = g dn ds
         decl (GDataInsDecl _ dn _ _ _ ds) = g dn ds
-        decl (DerivDecl _ _ hd) = (xs, xs) -- don't know whether this was on newtype or not
-            where xs = [ir hd]
-        decl _ = ([], [])
+        decl (DerivDecl _ _ hd) = mempty{derivesStandalone=[ir hd]}
+        decl _ = mempty
 
-        g dn ds = if isNewType dn then (xs,[]) else ([],xs)
+        g dn ds = if isNewType dn then mempty{derivesNewType=xs} else mempty{derivesData=xs}
             where xs = maybe [] (map ir . fromDeriving) ds
 
         ir (IRule _ _ _ x) = ih x
@@ -226,7 +269,7 @@ un = undefined
 hasT t x = not $ null (universeBi x `asTypeOf` [t])
 hasT2 ~(t1,t2) = hasT t1 & hasT t2
 
-hasS :: Biplate x (f S) => (f S -> Bool) -> x -> Bool
+hasS :: (Data x, Data (f S)) => (f S -> Bool) -> x -> Bool
 hasS test = any test . universeBi
 
 has f = any f . universeBi
