@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternGuards, ViewPatterns, RelaxedPolyRec, RecordWildCards, FlexibleContexts, ScopedTypeVariables, TupleSections #-}
+{-# LANGUAGE PatternGuards, ViewPatterns, RecordWildCards, FlexibleContexts, ScopedTypeVariables #-}
 
 {-
 The matching does a fairly simple unification between the two terms, treating
@@ -43,7 +43,6 @@ import Control.Applicative
 import Data.List.Extra
 import Data.Maybe
 import Data.Data
-import Unsafe.Coerce
 import Config.Type
 import Hint.Type
 import Control.Monad
@@ -163,8 +162,8 @@ nmOp nm  _ _ = False
 
 -- unify a b = c, a[c] = b
 unify :: Data a => NameMatch -> Bool -> a -> a -> Maybe [(String,Exp_)]
-unify nm root x y | Just x <- cast x = unifyExp nm root x (unsafeCoerce y)
-                  | Just x <- cast x = unifyPat nm x (unsafeCoerce y)
+unify nm root x y | Just (x,y) <- cast (x,y) = unifyExp nm root x y
+                  | Just (x,y) <- cast (x,y) = unifyPat nm x y
                   | Just (x :: S) <- cast x = Just []
                   | otherwise = unifyDef nm x y
 
