@@ -1,4 +1,4 @@
-{-# LANGUAGE TupleSections, PatternGuards, RecordWildCards #-}
+{-# LANGUAGE PatternGuards, RecordWildCards #-}
 
 -- | /WARNING: This module represents the evolving second version of the HLint API./
 --   /It will be renamed to drop the "3" in the next major version./
@@ -30,15 +30,13 @@ module Language.Haskell.HLint3(
     ) where
 
 import Config.Type
-import Config.Haskell as Settings hiding (findSettings)
+import Config.All
 import Idea
 import Apply
 import HLint
 import Hint.Type
 import Hint.All
 import CmdLine
-import Util
-import System.IO
 import Paths_hlint
 
 import Data.List.Extra
@@ -77,9 +75,8 @@ argsSettings args = do
         CmdMain{..} -> do
             -- FIXME: Two things that could be supported (but aren't) are 'cmdGivenHints' and 'cmdWithHints'.
             (fixities, classify, hints) <- findSettings (readSettingsFile $ Just cmdDataDir) Nothing
-            encoding <- if cmdUtf8 then return utf8 else readEncoding cmdEncoding
             let flags = parseFlagsSetLanguage (cmdExtensions cmd) $ parseFlagsAddFixities fixities $
-                        defaultParseFlags{cppFlags = cmdCpp cmd, encoding = encoding}
+                        defaultParseFlags{cppFlags = cmdCpp cmd}
             let ignore = [Classify Ignore x "" "" | x <- cmdIgnore]
             return (flags, classify ++ ignore, resolveHints hints)
         _ -> error "Can only invoke autoSettingsArgs with the root process"
@@ -93,6 +90,9 @@ argsSettings args = do
 --   argument, and all other files relative to the current directory.
 readSettingsFile :: Maybe FilePath -> String -> IO (FilePath, Maybe String)
 readSettingsFile dir x
+    | takeExtension x `elem` [".yml",".yaml"] = do
+        dir <- maybe getHLintDataDir return dir
+        return (dir </> x, Nothing)
     | Just x <- "HLint." `stripPrefix` x = do
         dir <- maybe getHLintDataDir return dir
         return (dir </> x <.> "hs", Nothing)
@@ -100,28 +100,14 @@ readSettingsFile dir x
 
 
 -- | Given a function to load a module (typically 'readSettingsFile'), and a module to start from
---   (defaults to @HLint.HLint@) find the information from all settings files.
+--   (defaults to @hlint.yaml@) find the information from all settings files.
 findSettings :: (String -> IO (FilePath, Maybe String)) -> Maybe String -> IO ([Fixity], [Classify], [Either HintBuiltin HintRule])
 findSettings load start = do
-    (file,contents) <- load $ fromMaybe "HLint.HLint" start
-    let flags = addInfix defaultParseFlags
-    res <- parseModuleEx flags file contents
-    case res of
-        Left (ParseError sl msg err) ->
-            error $ "Settings parse failure at " ++ showSrcLoc sl ++ ": " ++ msg ++ "\n" ++ err
-        Right (m, _) -> do
-            imported <- sequence [f $ fromNamed $ importModule i | i <- moduleImports m, importPkg i `elem` [Just "hint", Just "hlint"]]
-            let (classify, rules) = Settings.readSettings m
-            let fixities = getFixity =<< moduleDecls m
-            return $ concatUnzip3 $ (fixities,classify,map Right rules) : imported
-    where
-        builtins =  [(drop 4 $ show h, h :: HintBuiltin) | h <- [minBound .. maxBound]]
-
-        f x | x == "HLint.Builtin.All" = return ([], [], map Left [minBound..maxBound])
-        f x | Just x <- "HLint.Builtin." `stripPrefix` x = case lookup x builtins of
-                Just x -> return ([], [], [Left x])
-                Nothing -> error $ "Unknown builtin hints: HLint.Builtin." ++ x
-            | otherwise = findSettings load (Just x)
+    (file,contents) <- load $ fromMaybe "hlint.yaml" start
+    xs <- readFilesConfig [(file,contents)]
+    return ([x | Infix x <- xs]
+           ,[x | SettingClassify x <- xs]
+           ,[Right x | SettingMatchExp x <- xs] ++ map Left [minBound..maxBound])
 
 
 -- | Snippet from the documentation, if this changes, update the documentation
